/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#include "wallpaper.h"
#include "ui_wallpaper.h"
#include "pictureunit.h"
#include "maskwidget.h"

#include <QDBusReply>
#include <QDBusInterface>
#include <QDebug>
#include <QDesktopServices>
#include <QProcess>
#include <QFileInfo>
#include <QFileSystemWatcher>
#include "addbtn.h"
#include "common.h"

const QString kylinUrl = "https://www.ubuntukylin.com/wallpaper.html";

enum{
    PICTURE,   // 图片背景
    COLOR,     // 纯色背景
    SLIDESHOW  // 幻灯片背景
};


#define DEFAULT_BACKGROUND "/usr/share/backgrounds/1-warty-final-ubuntukylin.jpg"

#define ITEMWIDTH 182
#define ITEMHEIGH 126

#define COLORITEMWIDTH 56
#define COLORITEMHEIGH 56

Wallpaper::Wallpaper() : mFirstLoad(true),
                         initBackgroundFilename(""),
                         readBackgroundBus(nullptr),
                         useriFace(nullptr)
{
    pluginName = tr("Background");
    pluginType = PERSONALIZED;
    prePicUnit = nullptr;
    pObject = nullptr;
    pThread = nullptr;
}

Wallpaper::~Wallpaper()
{
    if (pObject) {
        pObject->setExit(true);
        delete pObject;
        pObject = nullptr;
    }
    if (pThread) {
        pThread->terminate();
        pThread->quit(); // 退出事件循环
        pThread->wait(); // 释放资源
        delete pThread;
        pThread = nullptr;
    }
    if (!mFirstLoad) {
        delete ui;
        ui = nullptr;
        delete xmlhandleObj;
        xmlhandleObj = nullptr;
    }
}

QString Wallpaper::plugini18nName() {
    return pluginName;
}

int Wallpaper::pluginTypes() {
    return pluginType;
}

QWidget *Wallpaper::pluginUi() {
    if (mFirstLoad) {
        mFirstLoad = false;

        ui = new Ui::Wallpaper;
        pluginWidget = new QWidget;
        pluginWidget->setAttribute(Qt::WA_DeleteOnClose);
        ui->setupUi(pluginWidget);
        initCenterUi();
        settingsCreate = false;
        initSearchText();
        // 初始化控件
        setupComponent();
        // 初始化gsettings
        const QByteArray id(BACKGROUND);
        if (QGSettings::isSchemaInstalled(id)){
            settingsCreate = true;

            bgsettings = new QGSettings(id, QByteArray(), this);
            initAccountDbus();
            initBgFormStatus();
            setupConnect();
            initBgOption();
            hideComponent();
        }
        // 构建xmlhandle对象
        xmlhandleObj = new XmlHandle();
        if (Common::isTablet()) {
            settingForIntel();    
        }
    } else {
        picFrame->resize(picFrame->size() - QSize(1,1));
        picFrame->resize(picFrame->size() + QSize(1,1));
    }
    return pluginWidget;
}

const QString Wallpaper::name() const {

    return QStringLiteral("Wallpaper");
}

bool Wallpaper::isShowOnHomePage() const
{
    return true;
}

QIcon Wallpaper::icon() const
{
    return QIcon::fromTheme("folder-pictures-symbolic");
}

bool Wallpaper::isEnable() const
{
    return true;
}

void Wallpaper::initSearchText() {
    //~ contents_path /Wallpaper/Desktop Background
    tr("Desktop Background");
    //~ contents_path /Wallpaper/Browse
    ui->browserLocalwpBtn->setText(tr("Browse"));
    //~ contents_path /Wallpaper/Online Picture
    ui->onlineBtn->setText(tr("Online Picture"));
    ui->onlineBtn->setFocusPolicy(Qt::NoFocus);
    ui->onlineBtn->setContentsMargins(0,0,0,0);
    ui->onlineBtn->setCursor( QCursor(Qt::PointingHandCursor));
    ui->onlineBtn->setStyleSheet("QPushButton{background: transparent;border-radius: 4px;text-decoration: underline;} ");
    connect(ui->onlineBtn, &QPushButton::clicked, this,[=] {
        QDesktopServices::openUrl(QUrl(QLatin1String("https://www.ubuntukylin.com/wallpaper.html")));
        Common::buriedSettings(name(), QString("onlineBtn"), QString("clicked"));
    });
    //~ contents_path /Wallpaper/Reset To Default
    ui->resetBtn->setText(tr("Reset To Default"));
}

void Wallpaper::setupComponent(){

    QString name = qgetenv("USER");
    if (name.isEmpty()) {
        name = qgetenv("USERNAME");
    }

    QString lockfilename = "/var/lib/lightdm-data/" + name + "/ukui-greeter.conf";
    mLockLoginSettings = new QSettings(lockfilename, QSettings::IniFormat, this);

    // 背景形式
    QStringList formList;
    formList << tr("picture") << tr("color")/* << tr("slideshow")*/ ;
    formWidget->comboBox()->addItem(formList.at(0), PICTURE);
    formWidget->comboBox()->addItem(formList.at(1), COLOR);


    // 图片背景
    picFlowLayout = new FlowLayout(picFrame,16, -1, -1);
    picFrame->setLayout(picFlowLayout);

    // 纯色背景
    colorFlowLayout = new FlowLayout(colorFrame,16, -1, -1);
    colorFrame->setLayout(colorFlowLayout);

    // 背景放置方式
    QStringList optionList;
    optionList << tr("scaled") << tr("wallpaper") << tr("centered") << tr("stretched") << tr("zoom") << tr("spanned");
    showModeWidget->comboBox()->addItem(optionList.at(0), "scaled");
    showModeWidget->comboBox()->addItem(optionList.at(1), "wallpaper");
    showModeWidget->comboBox()->addItem(optionList.at(2), "centered");
    showModeWidget->comboBox()->addItem(optionList.at(3), "stretched");
#ifndef MAVIS
    showModeWidget->comboBox()->addItem(optionList.at(4), "zoom");
    showModeWidget->comboBox()->addItem(optionList.at(5), "spanned");
#endif

    // 打开自定义颜色面板
    connect(colorAddButton, &AddBtn::clicked,[=](){
        colordialog = new ColorDialog(pluginWidget);
        connect(colordialog,&ColorDialog::colorSelected,this,&Wallpaper::colorSelectedSlot);
        colordialog->exec();
    });
}

void Wallpaper::setupConnect(){
    //使用线程构建本地壁纸文件；获取壁纸压缩QPixmap
    pThread = new QThread;
    pObject = new WorkerObject;

    connect(pObject, &WorkerObject::readWallPaper_data, this, [=](QStringList filenameList) {
        for (int i = 0; i < filenameList.size(); i++) {
            QString filename = filenameList.at(i);
            QString filanemaTemp = filename;
            filanemaTemp.replace("/","-");
            QString loadFilename = LOCAL_WALLPAPER_PATH + QString::number(i) + filanemaTemp;
            QPixmap pixmap;
            pixmap.load(loadFilename);
            createPictureUnit(pixmap, filename);
        }
    });

    connect(pObject, &WorkerObject::pixmapGenerate, this, [=](QPixmap pixmap, QString filename){
        createPictureUnit(pixmap, filename);
    });
    connect(pObject, &WorkerObject::workComplete, this, [=](QMap<QString, QMap<QString, QString>> wpInfoMaps){
        wallpaperinfosMap = wpInfoMaps;
    });

    pObject->moveToThread(pThread);
    connect(pThread, &QThread::started, pObject, &WorkerObject::run);
    connect(pThread, &QThread::finished, pObject, [=](){
        pThread->quit(); //退出事件循环
        pThread->wait(); //释放资源
        if (pThread) {
            delete pThread;
            pThread = nullptr;
        }
        if (pObject) {
            delete pObject;
            pObject = nullptr;
        }
    });

    pThread->start();

    connect(ui->browserLocalwpBtn, &QPushButton::clicked, [=]{
        showLocalWpDialog();
    });

    connect(ui->resetBtn, SIGNAL(clicked(bool)), this, SLOT(resetDefaultWallpaperSlot()));

    // 纯色背景
    QStringList colors;

    colors << "#2d7d9a" << "#018574" << "#107c10" << "#10893e" << "#038387" << "#486860" << "#525e54" << "#7e735f" << "#4c4a48" << "#000000";
    colors << "#ff8c00" << "#e81123" << "#d13438" << "#c30052" << "#bf0077" << "#9a0089" << "#881798" << "#744da9" << "#8764b8" << "#e9e9e9";

    for (QString color : colors){
        QPushButton * button = new QPushButton(colorFrame);
        button->setFixedSize(QSize(48, 48));
        QString btnQss = QString("QPushButton{background: %1; border: none; border-radius: 4px;}").arg(color);
        button->setStyleSheet(btnQss);

        connect(button, &QPushButton::clicked, [=] {
            QString widgetQss = QString("QWidget{background: %1; border-radius: 6px;}").arg(color);
            ui->previewWidget->setStyleSheet(widgetQss);
            changeGlobalTheme();
            // 设置系统纯色背景
            bgsettings->set(FILENAME, "");
            bgsettings->set(PRIMARY, QVariant(color));
            bgsettings->set(SECONDARY, QVariant(color));
            setLockBackground(color);

            ui->previewStackedWidget->setCurrentIndex(COLOR);
        });
        colorFlowLayout->addWidget(button);
    }

#if QT_VERSION <= QT_VERSION_CHECK(5, 12, 0)
    connect(formWidget->comboBox(), static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged), [=](int index){
#else
    connect(formWidget->comboBox(), QOverload<int>::of(&QComboBox::currentIndexChanged), [=](int index){
#endif
        Q_UNUSED(index)
        //切换
        int currentPage = formWidget->comboBox()->currentData(Qt::UserRole).toInt();
        if (currentPage == PICTURE) {
            colorGroup->setHidden(true);
            picFrame->setHidden(false);
            ui->bottomWidget->setHidden(false);
            Common::buriedSettings(name(), QString("formComBox"), QString("select"), QString("picture"));
        } else if (currentPage == COLOR){
            picFrame->setHidden(true);
            colorGroup->setHidden(false);
            ui->bottomWidget->setHidden(true);
            Common::buriedSettings(name(), QString("formComBox"), QString("select"),  QString("color"));
        }
    });

    connect(bgsettings, &QGSettings::changed, [=](QString key) {
        QString curPicname = bgsettings->get(key).toString();
        initBgFormStatus(true);
        if (key != "pictureOptions" && formWidget->comboBox()->currentIndex() == PICTURE) {
            QString filename = bgsettings->get(FILENAME).toString();
            setClickedPic(filename);
        }
        if (key == "pictureOptions") {
            QString option = bgsettings->get(OPTIONS).toString();
            int index = showModeWidget->comboBox()->findData(option);
            showModeWidget->comboBox()->blockSignals(true);
            showModeWidget->comboBox()->setCurrentIndex(index);
            showModeWidget->comboBox()->blockSignals(false);
            ui->previewLabel->setMode(option);
            ui->previewLabel->update();
        }
        //GSettings key picture-filename 这里收到 pictureFilename的返回值
        if (!QString::compare(key, "pictureFilename")) {
            if (curPicname == "") {
                if (prePicUnit != nullptr) {
                    prePicUnit->changeClickedFlag(false);
                    prePicUnit->setStyleSheet("border-width: 0px;");
                }
            }

            QDBusMessage msg = useriFace->call("SetBackgroundFile", curPicname);
            if (!msg.errorMessage().isEmpty())
                qDebug() << "update user background file error: " << msg.errorMessage();
        }
    });
}

int Wallpaper::_getCurrentBgForm(){
    QString filename = bgsettings->get(FILENAME).toString();

    int current = 0;

    // 设置当前背景形式
    if (filename.isEmpty()){
        current = COLOR;
    } else if (filename.endsWith("xml")){
        current = SLIDESHOW;
    } else {
        current = PICTURE;
    }

    return current;
}

void Wallpaper::initBgFormStatus(bool gsettingChange){
    initPreviewStatus(gsettingChange);

    int currentIndex = _getCurrentBgForm();
    //设置当前背景形式
    formWidget->comboBox()->blockSignals(true);
    formWidget->comboBox()->setCurrentIndex(currentIndex);
    formWidget->comboBox()->blockSignals(false);
    if (formWidget->comboBox()->currentIndex() == PICTURE) {
        colorGroup->setHidden(true);
        picFrame->setHidden(false);
        ui->bottomWidget->setHidden(false);
    } else if (formWidget->comboBox()->currentIndex() == COLOR){
        picFrame->setHidden(true);
        colorGroup->setHidden(false);
        ui->bottomWidget->setHidden(true);
    }

    ui->previewStackedWidget->setCurrentIndex(currentIndex);

    //根据背景形式选择显示组件
    showComponent(currentIndex);
}

void Wallpaper::showComponent(int index) {
    Q_UNUSED(index);
}

void Wallpaper::setLockBackground(QString bg) {
    mLockLoginSettings->beginGroup("greeter");
    mLockLoginSettings->setValue("color", bg);
    mLockLoginSettings->endGroup();
}

void Wallpaper::initBgOption()
{
    QString option = bgsettings->get(OPTIONS).toString();
    int index = showModeWidget->comboBox()->findData(option);
    showModeWidget->comboBox()->setCurrentIndex(index);
    ui->previewLabel->setMode(option);
    ui->previewLabel->update();
    connect(showModeWidget->comboBox(), QOverload<int>::of(&QComboBox::currentIndexChanged), this, [=](int index) {
        bgsettings->set(OPTIONS, showModeWidget->comboBox()->itemData(index).toString());
        Common::buriedSettings(name(), QString("showModeComboBox"), QString("select"), showModeWidget->comboBox()->itemData(index).toString());
    });
}

void Wallpaper::hideComponent()
{
    ui->onlineBtn->setHidden(Common::isZJY());
}

void Wallpaper::initPreviewStatus(bool gsettingChange){
    // 设置图片背景的预览效果
    if (!gsettingChange && readBackgroundBus && readBackgroundBus->isValid()) {
        QDBusReply<QVariant> msg = readBackgroundBus->call("Get", "org.freedesktop.Accounts.User", "BackgroundFile");
        if (msg.isValid()) {
            initBackgroundFilename = msg.value().toString();
        }
    }
    if (initBackgroundFilename.isEmpty() || gsettingChange) {
        initBackgroundFilename = bgsettings->get(FILENAME).toString();
    }

    if (!QFile::exists(initBackgroundFilename) || initBackgroundFilename == "/usr/share/backgrounds/ubuntukylin-default-settings.jpg") {
        initBackgroundFilename = DEFAULT_BACKGROUND;
    }

    QFileInfo fileinfo = QFileInfo(initBackgroundFilename);
    if(fileinfo.suffix() == "dib") {
        QFile::copy(initBackgroundFilename,".TEMP.bmp");
        ui->previewLabel->setLocalPixmap(QPixmap(".TEMP.bmp"));
    } else {
        ui->previewLabel->setLocalPixmap(QPixmap(initBackgroundFilename));
    }
    ui->previewLabel->update();
    // 设置纯色背景的预览效果
    QString color = bgsettings->get(PRIMARY).toString();
    if (!color.isEmpty()){
        QString widgetQss = QString("QWidget{background: %1;}").arg(color);
        ui->previewWidget->setStyleSheet(widgetQss);
    }
}

// 自定义颜色面板选定颜色
void Wallpaper::colorSelectedSlot(QColor color){
    qDebug() << "colorSelectedSlot" << color << color.name();

    QString widgetQss = QString("QWidget{background: %1;}").arg(color.name());
    ui->previewWidget->setStyleSheet(widgetQss);

    // 设置系统纯色背景
    bgsettings->set(FILENAME, "");
    bgsettings->set(PRIMARY, QVariant(color.name()));
    setLockBackground(color.name());

    ui->previewStackedWidget->setCurrentIndex(COLOR);
}

void Wallpaper::wpOptionsChangedSlot(QString op){
    Q_UNUSED(op)
    //将改动保存至文件
    xmlhandleObj->xmlUpdate(wallpaperinfosMap);
}

void Wallpaper::setModeldata(){
    QMap<QString, QMap<QString, QString> >::iterator iters = wallpaperinfosMap.begin();
    for (int row = 0; iters != wallpaperinfosMap.end(); iters++, row++){
        if (QString(iters.key()) == "head") //跳过xml的头部信息
            continue;
        QMap<QString, QString> wpMap = (QMap<QString, QString>)iters.value();
        QString delstatus = QString(wpMap.find("deleted").value());
        if (delstatus == "true") //跳过被删除的壁纸
            continue;

        QString filename = QString(iters.key());
        QPixmap pixmap(filename);

        wpListModel.insertRows(row, 1, QModelIndex());
        QModelIndex wpindex = wpListModel.index(row, 0, QModelIndex());
        wpListModel.setData(wpindex, QIcon(pixmap.scaled(QSize(160,100))), Qt::DecorationRole);
        wpListModel.setData(wpindex, QString("%1\nfolder: %2\n").arg(wpMap.find("name").value()).arg(filename), Qt::ToolTipRole);
    }
}

void Wallpaper::resetDefaultWallpaperSlot(){
    GSettings * wpgsettings;
    wpgsettings = g_settings_new(BACKGROUND);
    GVariant * variant = g_settings_get_default_value(wpgsettings, FILENAME);
    gsize size = g_variant_get_size(variant);
    const char * dwp = g_variant_get_string(variant, &size);
    g_object_unref(wpgsettings);

    bgsettings->set(FILENAME, QVariant(QString(dwp)));
    setClickedPic(QString(dwp));
    Common::buriedSettings(name(), QString("resetBtn"), QString("clicked"));
}

void Wallpaper::showLocalWpDialog(){
    QStringList filters;
    filters<<tr("Wallpaper files(*.jpg *.jpeg *.bmp *.dib *.png *.jfif *.jpe *.gif *.tif *.tiff *.wdp)");
    QFileDialog fd(pluginWidget);
    QList<QUrl> usb_list = fd.sidebarUrls();
    int sidebarNum = 8;// 最大添加U盘数，可以自己定义
    QString home_path = QDir::homePath().section("/", -1, -1);
    QString mnt = "/media/" + home_path + "/";
    QDir mntDir(mnt);
    mntDir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
    QFileInfoList file_list = mntDir.entryInfoList();
    QList<QUrl> mntUrlList;
    for (int i = 0; i < sidebarNum && i < file_list.size(); ++i) {
        QFileInfo fi = file_list.at(i);
        mntUrlList << QUrl("file://" + fi.filePath());
    }

    QFileSystemWatcher m_fileSystemWatcher(&fd);
    m_fileSystemWatcher.addPath("/media/" + home_path + "/");
    connect(&m_fileSystemWatcher, &QFileSystemWatcher::directoryChanged, &fd,
            [=, &sidebarNum, &mntUrlList, &usb_list, &fd](const QString path) {
        QDir m_wmntDir(path);
        m_wmntDir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
        QFileInfoList m_wfilist = m_wmntDir.entryInfoList();
        mntUrlList.clear();
        for (int i = 0; i < sidebarNum && i < m_wfilist.size(); ++i) {
            QFileInfo m_fi = m_wfilist.at(i);
            mntUrlList << QUrl("file://" + m_fi.filePath());
        }
        fd.setSidebarUrls(usb_list + mntUrlList);
        fd.update();
    });

    connect(&fd, &QFileDialog::finished, &fd, [=, &usb_list, &fd]() {
        fd.setSidebarUrls(usb_list);
    });

    fd.setDirectory(QString(const_cast<char *>(g_get_user_special_dir(G_USER_DIRECTORY_PICTURES))));
    fd.setAcceptMode(QFileDialog::AcceptOpen);
    fd.setViewMode(QFileDialog::List);
    fd.setNameFilters(filters);
    fd.setFileMode(QFileDialog::ExistingFile);
    fd.setWindowTitle(tr("select custom wallpaper file"));
    fd.setLabelText(QFileDialog::Accept, tr("Select"));
    fd.setLabelText(QFileDialog::LookIn, tr("Position: "));
    fd.setLabelText(QFileDialog::FileName, tr("FileName: "));
    fd.setLabelText(QFileDialog::FileType, tr("FileType: "));
    fd.setLabelText(QFileDialog::Reject, tr("Cancel"));

    fd.setSidebarUrls(usb_list + mntUrlList);

    if (fd.exec() != QDialog::Accepted)
        return;
    QString selectedfile;
    selectedfile = fd.selectedFiles().first();

    QStringList fileRes = selectedfile.split("/");

    QProcess * process = new QProcess();
    QString program("cp");
    QStringList arguments;
    arguments << selectedfile ;
    arguments << "/tmp";
    process->start(program, arguments);

    QString bgfile = "/tmp/" + fileRes.at(fileRes.length() - 1);

    // TODO: chinese and space support
    bgsettings->set(FILENAME, selectedfile);
    if (prePicUnit != nullptr) {  //去掉选定标记
        prePicUnit->changeClickedFlag(false);
        prePicUnit->setStyleSheet("border-width: 0px;");
    }
    Common::buriedSettings(name(), QString("browserLocalwpBtn"), QString("clicked"));
}

void Wallpaper::add_custom_wallpaper(){
    QString filters = "Wallpaper files(*.png *.jpg)";
    QFileDialog fd(pluginWidget);

    fd.setDirectory(QString(const_cast<char *>(g_get_user_special_dir(G_USER_DIRECTORY_PICTURES))));
    fd.setAcceptMode(QFileDialog::AcceptOpen);
    fd.setViewMode(QFileDialog::List);
    fd.setNameFilter(filters);
    fd.setFileMode(QFileDialog::ExistingFile);
    fd.setWindowTitle(tr("select custom wallpaper file"));
    fd.setLabelText(QFileDialog::Accept, tr("Select"));
    fd.setLabelText(QFileDialog::LookIn, tr("Position: "));
    fd.setLabelText(QFileDialog::FileName, tr("FileName: "));
    fd.setLabelText(QFileDialog::FileType, tr("FileType: "));
    fd.setLabelText(QFileDialog::Reject, tr("Cancel"));


    QString selectedfile;
    selectedfile = fd.selectedFiles().first();

    if (wallpaperinfosMap.contains(selectedfile)){
        wallpaperinfosMap[selectedfile]["deleted"] = "false";
    } else {
        QMap<QString, QString> tmpinfo;
        tmpinfo.insert("artist", "(none)");
        tmpinfo.insert("deleted", "false");
        tmpinfo.insert("filename", selectedfile);
        tmpinfo.insert("name", selectedfile.split("/").last());
        tmpinfo.insert("options", "zoom");
        tmpinfo.insert("pcolor", "#000000");
        tmpinfo.insert("scolor", "#000000");
        tmpinfo.insert("shade_type", "solid");
        wallpaperinfosMap.insert(selectedfile, tmpinfo);

    }
    xmlhandleObj->xmlUpdate(wallpaperinfosMap);
}

void Wallpaper::del_wallpaper(){
    // 将改动保存至文件
    xmlhandleObj->xmlUpdate(wallpaperinfosMap);
}

void Wallpaper::setClickedPic(QString fileName) {
    if (prePicUnit != nullptr) {
        prePicUnit->changeClickedFlag(false);
        prePicUnit->setStyleSheet("border-width: 0px;");
    }
    for (int i = picFlowLayout->count() - 1; i >= 0; --i) { 
        QLayoutItem *it      = picFlowLayout->itemAt(i);
        PictureUnit *picUnit = static_cast<PictureUnit*>(it->widget());
        if (fileName == picUnit->filenameText()) {
            picUnit->changeClickedFlag(true);
            prePicUnit = picUnit;
            picUnit->setFrameShape(QFrame::Box);
            picUnit->setStyleSheet(picUnit->clickedStyleSheet);
        }
    }
}

void Wallpaper::settingForIntel()
{
    formWidget->setVisible(false);
    ui->onlineBtn->setVisible(false);
    ui->resetBtn->setFixedHeight(48);;
    ui->browserLocalwpBtn->setFixedHeight(48);
}

void Wallpaper::initAccountDbus()
{
        //壁纸变动后改变用户属性
    QDBusInterface * interface = new QDBusInterface("org.freedesktop.Accounts",
                                                     "/org/freedesktop/Accounts",
                                                     "org.freedesktop.Accounts",
                                                     QDBusConnection::systemBus(),
                                                     this);

    if (!interface->isValid()){
        qCritical() << "Create /org/freedesktop/Accounts Client Interface Failed " << QDBusConnection::systemBus().lastError();
        return;
    }

    QDBusReply<QDBusObjectPath> reply =  interface->call("FindUserByName", g_get_user_name());
    QString userPath;
    if (reply.isValid()) {
        userPath = reply.value().path();
    } else {
        qCritical() << "Call 'GetComputerInfo' Failed!" << reply.error().message();
        return;
    }

    useriFace = new QDBusInterface("org.freedesktop.Accounts",
                                                    userPath,
                                                    "org.freedesktop.Accounts.User",
                                                    QDBusConnection::systemBus(),
                                                    this);
    if (!useriFace->isValid()){
        qCritical() << QString("Create %1 Client Interface Failed").arg(userPath) << QDBusConnection::systemBus().lastError();
        return;
    }
    readBackgroundBus = new QDBusInterface("org.freedesktop.Accounts",
                                            userPath,
                                            "org.freedesktop.DBus.Properties",
                                            QDBusConnection::systemBus(),
                                            this);
    if (!readBackgroundBus->isValid()){
        qCritical() << QString("Create %1 Client Interface Failed").arg(userPath) << QDBusConnection::systemBus().lastError();
        return;
    }

}

void Wallpaper::createPictureUnit(QPixmap pixmap, QString filename)
{
    PictureUnit * picUnit = new PictureUnit(this->pluginWidget);
    picUnit->setPixmap(pixmap);
    picUnit->setFilenameText(filename);
    if (initBackgroundFilename == filename) {
        if (prePicUnit != nullptr) {
            prePicUnit->changeClickedFlag(false);
            prePicUnit->setStyleSheet("border-width: 0px;");
        }
        picUnit->changeClickedFlag(true);
        prePicUnit = picUnit;
        picUnit->setFrameShape(QFrame::Box);
        picUnit->setStyleSheet(picUnit->clickedStyleSheet);
    }

    connect(picUnit, &PictureUnit::clicked, [=](QString fn){
        if (prePicUnit != picUnit && prePicUnit != nullptr) {
            changeGlobalTheme();
            prePicUnit->changeClickedFlag(false);
            prePicUnit->setStyleSheet("border-width: 0px;");
        }
        picUnit->changeClickedFlag(true);
        prePicUnit = picUnit;
        picUnit->setFrameShape(QFrame::Box);
        picUnit->setStyleSheet(picUnit->clickedStyleSheet);
        bgsettings->set(FILENAME, fn);
        setLockBackground("");
        ui->previewStackedWidget->setCurrentIndex(PICTURE);
        Common::buriedSettings(name(), QString("picUnit"), QString("clicked"), fn);
    });

    picFlowLayout->addWidget(picUnit);
    picNum++;
}

void Wallpaper::changeGlobalTheme()
{
    if (QGSettings::isSchemaInstalled(QByteArray("org.ukui.globaltheme.settings"))) {
        QGSettings globalTheme("org.ukui.globaltheme.settings");
        globalTheme.set("global-theme-name", "custom");
    }
}

void Wallpaper::initCenterUi()
{
    centerGroup     = new SettingGroup(this->pluginWidget);
    formWidget      = new ComboxWidget(tr("Background"), centerGroup);
    showModeWidget  = new ComboxWidget(tr("Mode"), centerGroup);
    colorGroup      = new SettingGroup(centerGroup);
    addbuttonFrame  = new UkccFrame(colorGroup);
    addbuttonlayout = new QHBoxLayout(addbuttonFrame);
    colorAddButton  = new AddButton(addbuttonFrame);
    colorFrame      = new UkccFrame(colorGroup);
    picFrame        = new UkccFrame(centerGroup);

    addbuttonlayout  -> setMargin(0);
    formWidget       -> setTitleFixedWidth(172);
    showModeWidget   -> setTitleFixedWidth(172);
    ui->centerLayout -> addWidget(centerGroup);
    centerGroup      -> addWidget(formWidget);
    centerGroup      -> addWidget(showModeWidget);
    centerGroup      -> addWidget(colorGroup);
    centerGroup      -> addWidget(picFrame);
    colorGroup       -> addWidget(colorFrame);
    colorGroup       -> addWidget(addbuttonFrame);
    addbuttonlayout  -> addWidget(colorAddButton);
    picFrame         -> setLineWidth(0);
}


